/*
 * Copyright 2008 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.derquinsej.collect;

import java.util.List;

/**
 * A hierarchy is a set of objects identified by a key organized in a filesystem
 * tree-like structure. Each level is ordered. Neither null keys nor Null nodes
 * are allowed.
 * @author Andres Rodriguez
 * @param <K> Type of the keys.
 * @param <V> Type of the elements.
 */
public interface Hierarchy<K, V> {
	/**
	 * Returns the first level of the hierarchy. That is, the elements that have
	 * no parent.
	 * @return The list of elements with no parent.
	 */
	List<V> getFirstLevel();

	/**
	 * Returns the children of the specified element.
	 * @param element The node which children are requested.
	 * @return The list of children, the empty list if it has no children.
	 * @throws NullPointerException if the specified element is {@code null}.
	 * @throws IllegalArgumentException if the specified element is not part of
	 *             the hierarchy.
	 */
	List<V> getChildren(V element);

	/**
	 * Returns the parent of the specified element.
	 * @param element The element wich parent is requested.
	 * @return The parent element or {@code null} if it is a first-level
	 *         element.
	 * @throws NullPointerException if the specified element is {@code null}.
	 * @throws IllegalArgumentException if the specified element is not part of
	 *             the hierarchy.
	 */
	V getParent(V element);

	/**
	 * Checks if the hierarchy contains the specified element.
	 * @param element Element to check
	 * @throws NullPointerException if the specified element is {@code null}.
	 * @return True if the hierarchy contains the element, false otherwise.
	 */
	boolean contains(V element);

	/**
	 * Returns the children of the element with the specified key.
	 * @param key The key of the element which children are requested.
	 * @return The list of children or the empty list if it has no children.
	 * @throws NullPointerException if the specified key is {@code null}.
	 * @throws IllegalArgumentException if no element with the specified key is
	 *             part of the hierarchy.
	 */
	List<V> getChildrenByKey(K key);

	/**
	 * Returns the parent of the element with the specified key.
	 * @param key The key of the element which parent is requested.
	 * @return The parent element or {@code null} if it is a first-level
	 *         element.
	 * @throws NullPointerException if the specified key is {@code null}.
	 * @throws IllegalArgumentException if no element with the specified key is
	 *             part of the hierarchy.
	 */
	V getParentByKey(K key);

	/**
	 * Checks if the hierarchy contains a element with the specified key.
	 * @param key Key to check
	 * @return True if the hierarchy contains the element, false otherwise..
	 * @throws NullPointerException if the specified key is {@code null}.
	 */
	boolean containsKey(K key);

	/**
	 * Returns the value with the specified key.
	 * @param key The key to search.
	 * @return The object with the specified key or {@code null} if it not found
	 *         or the provided key is {@code null};
	 * @throws NullPointerException if the specified id is {@code null}.
	 */
	V get(K key);
}
